var WIDGET_VERSION = "1.0";
var BASE_BLINK_PERIOD = 40;

var moonNum     = 1;
var lookNum     = 0;
var safeForWork = 1;

if (window.widget)
{
	widget.onshow = doShow;
	widget.onhide = doHide;
}

function doShow() { startBlink(); }
function doHide() { stopBlink();  }

function setup()
{
	showWidgetVersion();
	
	loadPreferences();
	
	createButtons();
	updateButtons();
}

//
// PREFERENCES
//

function showWidgetVersion()
{
	document.getElementById("versString").innerHTML = "v"+WIDGET_VERSION;
}

function loadPreferences()
{
	moonNum     = validatedPreferenceForKey( windowSpecificKey("moonNum"    ));
	lookNum     = validatedPreferenceForKey( windowSpecificKey("lookNum"    ));
	safeForWork = validatedPreferenceForKey( windowSpecificKey("safeForWork"));
}

function savePreferences()
{
	widget.setPreferenceForKey(     moonNum, windowSpecificKey("moonNum"    ) );
	widget.setPreferenceForKey(     lookNum, windowSpecificKey("lookNum"    ) );
	widget.setPreferenceForKey( safeForWork, windowSpecificKey("safeForWork") );
}

function windowSpecificKey   ( aKey ) { return aKey + "---" + widget.identifier; }
function nonWindowSpecificKey( aKey ) { return aKey.split("---")[0]; }

function validatedPreferenceForKey( aKey )
{
	var theResult = widget.preferenceForKey(aKey);
	
	if ( theResult == undefined )
		theResult = defaultPreferenceForKey( nonWindowSpecificKey( aKey  ) );
	
	return theResult;
}

function defaultPreferenceForKey( aKey )
{
	var theResult;
	
	switch ( aKey )
	{
		case "moonNum"    : theResult = 1; break;
		case "lookNum"    : theResult = 0; break;
		case "safeForWork": theResult = 1; break;
	}
	
	return theResult;
}


//
// BUTTONS
//

function createButtons()
{
	createGenericButton(document.getElementById("doneButton"), "Done", hidePrefs, 50);
}

function clickPopup( aPopupId, aSelIdx )
{
	var theType = aPopupId.split("-")[0];
	var theVal  = 1*document.getElementById(aPopupId).options[aSelIdx].value;
	
	switch ( theType )
	{
		case "moon": moonNum = theVal; break;
		case "look": lookNum = theVal; break;
	}
	
	updateButtons();
}

function clickRadio( aBtnId )
{
	var theFamily = aBtnId.split("-")[1];
	var theMember = aBtnId.split("-")[2];
	
	switch ( theFamily )
	{
		case "moon": moonNum = 1*theMember; break;
		case "look": lookNum = 1*theMember; break;
	}
	
	updateButtons();
}

function clickCheckBox( aBtnId )
{
	switch ( aBtnId.split("-")[1] )
	{
		case "sfw": safeForWork = 1 - safeForWork; break;
	}
	
	updateButtons();
}

function selectPopupItemWithValue( aPopupId, aVal )
{
	aPopupId = aPopupId+"-popup";
	
	var thePopup = document.getElementById(aPopupId);
	
	if ( thePopup )
	{
		var aIdx;
		
		for ( aIdx = 0; aIdx < thePopup.options.length; aIdx++ )
			thePopup.options[aIdx].selected = ( aVal == 1*thePopup.options[aIdx].value );
		
		aIdx = thePopup.selectedIndex;
		document.getElementById(aPopupId+"-text").innerHTML = ( aIdx > -1 ? thePopup.options[aIdx].innerHTML : "" );
	}
}

function updateButtons()
{
	selectPopupItemWithValue( "moon", moonNum );
	selectPopupItemWithValue( "look", lookNum );
	
	document.getElementById("check-sfw"   ).className = ( safeForWork == 1 ? "checkBoxOnn"    : "checkBoxOff"    );
	
	document.getElementById("flip"        ).className = ( "flip-"+moonNum+"-"+( lookNum > 0 ? 1 : 0 ) );
	document.getElementById("fliprollie"  ).className = ( "flip-"+moonNum+"-"+( lookNum > 0 ? 1 : 0 ) );
	
	var flipColor = "white";
	if ( lookNum == 0 )
		flipColor = "white";
		
		
	document.getElementById("flip"      ).style.background = "url(file:///System/Library/WidgetResources/ibutton/"+flipColor+     "_i.png) no-repeat top left";
	document.getElementById("fliprollie").style.background = "url(file:///System/Library/WidgetResources/ibutton/"+flipColor+"_rollie.png) no-repeat top left";

	
	var imageURL = "images/moon-";
	
	imageURL += moonNum+"-";
	imageURL += ( lookNum > 0 ? 1 : 0 ) + "-";
	imageURL += safeForWork;
	
	document.getElementById("moon-bg").style.background = "url("+imageURL+".png) center center no-repeat";
	document.getElementById("moon-fg").src              = imageURL+"-flasher.png";
	
	startBlink();
}


//
// BLINKING
//

var blinkAnimation = {timer:null, counter:0};

function stopBlink()
{
	clearInterval ( blinkAnimation.timer );
	blinkAnimation.timer   = null;
	blinkAnimation.counter = 0;
	
	document.getElementById("moon-fg").style.opacity = 0.0;
}

function startBlink()
{
	stopBlink();
	
	if ( lookNum > 1 )
	{
		blinkAnimation.timer   = setInterval ("blinkAnimate();", 13);
		blinkAnimate();
	}
}

function blinkAnimate()
{
	var theOpacity = 1.0;
	var thePeriod  = BASE_BLINK_PERIOD;
	
	switch ( lookNum )
	{
		case 2: thePeriod *= 2; break;
		case 4: thePeriod /= 2; break;
	}
	
	blinkAnimation.counter = ( 1 + blinkAnimation.counter ) % thePeriod;
	
	// fade-on, fade-off blinking ...
	//theOpacity = blinkAnimation.counter;
	//if ( theOpacity > (thePeriod/2) )
	//	theOpacity = thePeriod - theOpacity;
	//theOpacity = 2*theOpacity / thePeriod;

	theOpacity = 1.0;
	if ( blinkAnimation.counter > thePeriod/2 )
		theOpacity = 0.0;
	
	document.getElementById("moon-fg").style.opacity = theOpacity;
}



//
// FLIPPING
//

function showPrefs()
{
	var front = document.getElementById("front");
	var back  = document.getElementById("back" );
	
	
	if (window.widget)
		widget.prepareForTransition("ToBack");
	
	
	front.style.display = "none";
	back.style.display  = "block";
	
	updateButtons();
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);
}

function hidePrefs()
{
	var front = document.getElementById("front");
	var back  = document.getElementById("back" );
	
	back.style.display  = "none";
	front.style.display = "block";
	
	updateButtons();
	
	if (window.widget)
		widget.prepareForTransition("ToFront");
	
	document.getElementById("fliprollie").style.display = "none";
	savePreferences();
	
	if (window.widget)
		setTimeout ('widget.performTransition();', 0);
}

function enterflip(event) { document.getElementById('fliprollie').style.display = 'block'; }
function exitflip (event) { document.getElementById('fliprollie').style.display = 'none'; }

var flipShown = false;
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, firstElement:null, timer:null};
function mousemove (event)
{
	if (!flipShown)
	{
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13;
		
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 1.0;
		animate();
		flipShown = true;
	}
}
function mouseexit (event)
{
	if (flipShown)
	{
		// fade in the info button
		if (animation.timer != null)
		{
			clearInterval (animation.timer);
			animation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - 13;
		
		animation.duration = 500;
		animation.starttime = starttime;
		animation.firstElement = document.getElementById ('flip');
		animation.timer = setInterval ("animate();", 13);
		animation.from = animation.now;
		animation.to = 0.0;
		animate();
		flipShown = false;
	}
}

function animate()
{
	var T;
	var ease;
	var time = (new Date).getTime();
	
	T = limit_3(time-animation.starttime, 0, animation.duration);
	
	if (T >= animation.duration)
	{
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat (animation.from, animation.to, ease);
	}
	
	animation.firstElement.style.opacity = animation.now;
}

function limit_3 (a, b, c)                 { return a < b ? b : (a > c ? c : a); }
function computeNextFloat (from, to, ease) { return from + (to - from) * ease;   }